"use strict";
this.name        = "escortdeck_escort";
this.author      = "Norby";
this.copyright   = "2014 Norbert Nagy";
this.licence     = "CC BY-NC-SA 4.0";
this.description = "AI support functions for escorts";

this._landing = function() {this._landing2(this.ship);}

this._landing2 = function(ship) {
	var w = worldScripts.escortdeck;
	var p = player.ship;
	if( !p || !p.isValid ) return;
	ship.target = p;
	ship.savedCoordinates = p.position; // must use setDestinationToTarget also
	var d = ship.position.distanceTo(p.position);
	if( d < w.$EscortDeckLandingDist ) { //enough near?: add to the deck
		var pad = w.$EscortDeckShip.indexOf(ship);
		if( pad > -1 && !w.$EscortDeck_PutShip(ship, pad, p, w, true, true) )
			ship.AIState == "DECK"; //landed
		else ship.AIState == "FOLLOW"; //no room or replaced with another ship
//		if(w.$Debug) log(this.name, ship.displayName+" pad:"+pad+" "+ship.AIState); //debug
	} else { //do not attack hostiles during landing
		if( d < 1.5 * w.$EscortDeckLandingDist ) ship.reactToAIMessage("PLAYER_NEXT");
		else if( d < 2500 && p.speed < ship.maxSpeed)
			ship.reactToAIMessage("PLAYER_NEAR");
		else ship.reactToAIMessage("PLAYER_FAR");
	}
}

this._locatePlayer = function() {this._locatePlayer2(this.ship);}

this._locatePlayer2 = function(ship) {
	var p = player.ship;
	if( !p || !p.isValid ) return;
	var sc = ship.script;
	ship.target = p;
	ship.savedCoordinates = p.position;
	var d = ship.position.distanceTo(p.position);
	if( player.alertCondition == 3 && sc._inEscortFleet2(ship) ) {
		 //if not in fleet then follow but not defend the player, just himself
		ship.AIState == "GOTO"; //attack hostiles
	} else {
		if( d < 500 ) ship.reactToAIMessage("PLAYER_NEXT");
		else if( d < 2500 && p.speed < ship.maxSpeed)
			ship.reactToAIMessage("PLAYER_NEAR");
		else ship.reactToAIMessage("PLAYER_FAR");
	}
}

this._combatCheck = function() {this._combatCheck2(this.ship);}

this._combatCheck2 = function(ship) {
	var p = player.ship;
	if( !p || !p.isValid ) return;
	var sc = ship.script;

	if(ship.target && ship.target.isDerelict)
	{
		ship.target = null;
		ship.reactToAIMessage("TARGET_EJECTED");
	}
	
	var t = sc._playerTarget2(ship);
	if( !ship.target || !ship.target.isValid
		|| t && t.isValid && ship.target != t ) {
		ship.AIState = "GOTO"; //find new target or player set new target
		return;
	}
	
	if( ship.position.distanceTo(p.position) > 24000 ) {
		// if escort is far from the player, break off combat
		if( ship.target && ship.target != sc._findPlayerNearestHostile2(ship) ) {
			ship.target = null;
			ship.AIState = "GOTO"; //change target to the player's nearest hostile
		}
	} else {
		var e = ship.energy;
		var m = ship.maxEnergy;
		ship.savedCoordinates = ship.target.position;
		if( e < m * 0.4 ) //cautious: flee if energy is below 40%
			ship.AIState="FLEE"; //avoid firing ECM
		else if( sc._Inject2(ship.target,ship) ) { //inject?
			if( ship.AIState == "INJECT" )
				ship.reactToAIMessage("ENEMY_FIRE");
			else ship.AIState == "INJECT"; //switch to inject
		} else {
			if( ship.AIState == "ATTACK" )
				ship.reactToAIMessage("ENEMY_FIRE");
			else ship.AIState == "ATTACK"; //no more inject
		}
	}
}

this._findPlayerHostiles = function() {this._findPlayerHostiles2(this.ship);}

this._findPlayerHostiles2 = function(ship) {
	var p = player.ship;
	if( !p || !p.isValid ) return;
	var sc = ship.script;

	if( !sc._inEscortFleet2(ship) ) { //follow but not defend the player, just himself
		ship.target = null;
		ship.reactToAIMessage("NO_HOSTILE_FOUND");
	}
	var t = sc._playerTarget2(ship); //follow target changes of player
	if( t && t.isValid ) sc._foundPlayerHostile2( t, ship );
	else if( ship.target && ship.target.isValid && sc._isHostileToPlayer(ship.target) ) {
		sc._foundPlayerHostile2( ship.target, ship ); //keep the current target
	} else {
		var hostile = sc._findPlayerNearestHostile2(ship);
		if( hostile ) sc._foundPlayerHostile2( hostile, ship );
		else {
			ship.target = null;
			ship.reactToAIMessage("NO_HOSTILE_FOUND");
		}
	}
}

this._findPlayerNearestHostile = function() {this._findPlayerNearestHostile2(this.ship);}

this._findPlayerNearestHostile2 = function(ship) {
	var sc = ship.script;
	var hostile = null;
	var targets = ship.checkScanner(true);//much faster than filteredEntities
//	var targets = system.filteredEntities(sc, sc._isHostileToPlayer, ship, ship.scannerRange);
//	if( targets.length > 0 ) hostile = targets[0];
	if( targets && targets.length > 0 )
		for( var i = 0; !hostile && i < targets.length; i++ ) {
			var t = targets[i];
			if( t && t.isValid && sc._isHostileToPlayer( t ) ) {
				hostile = t;
			}
		}
	return(hostile);
}

this._fireCheck = function() {this._fireCheck2(this.ship);}

this._fireCheck2 = function(ship) {
	var t = ship.target;
	if( !t || !t.isValid || t.isPlayer || (t.script && ( t.script.$EscortDeckUsable > 0
		|| t.script.$CarriersShipStorageID > -1 ) ) ) {
		ship.target = null;
		ship.reactToAIMessage("FRIENDLY_FIRE");
	} else ship.reactToAIMessage("ENEMY_FIRE");
}

this._fleeCheck = function() {this._fleeCheck2(this.ship);}

this._fleeCheck2 = function(ship) {
	var e = ship.energy;
	var m = ship.maxEnergy;
	if( (!ship.target || !ship.target.isMissile) && e > m * 0.8 ) {
		//not a missile run and recovered 80% energy
		ship.AIState = "GOTO";
	}
}

this._foundPlayerHostile = function( hostile ) {this._foundPlayerHostile2(hostile, this.ship);}

this._foundPlayerHostile2 = function( hostile, ship ) {
	var sc = ship.script;

	if( !hostile || !hostile.isValid) {
		ship.AIState = "FOLLOW"; //find new target
		return;
	}
	
	ship.target = hostile;
	ship.savedCoordinates = hostile.position;
	var sniper = false;
	if( sc._isSniper2(ship) ) ship.reactToAIMessage("SNIPING");
	else if(sc._Inject2(hostile, ship))
		ship.reactToAIMessage("HOSTILE_FAR"); //inject
	else ship.reactToAIMessage("HOSTILE_FOUND"); //attack
}

this._inEscortFleet = function() {this._inEscortFleet2(this.ship);}

this._inEscortFleet2 = function(ship) { //if not then follow but not defend the player, just himself
	var w = worldScripts.escortdeck;
	if( w.$EscortDeckShip.indexOf(ship) > -1 ) return true;
	return false;
}

this._isHostileToPlayer = function(entity) {
	return (entity.isThargoid || (entity.isShip && !entity.isPlayer && entity.target
		&& !entity.isDerelict
		&& ( entity.target == player.ship && entity.hasHostileTarget
			|| entity.script && entity.script.$EscortDeck_Target )
		&& ( !entity.script || !entity.script.$CarriersShipStorageID //is not owned by player
			|| entity.script.$CarriersShipStorageID == -1
			|| !( entity.script.$EscortDeckUsable > 0 ))));
}

this._isSniper = function() {this._isSniper2(this.ship);}

this._isSniper2 = function(ship) {
	var l = ship.forwardWeapon.equipmentKey;
	if( l == "EQ_WEAPON_MILITARY_LASER" ) return true;
	else if( l ) {  //Oolite 1.81 can define custom lasers
		var i = EquipmentInfo.infoForKey(l);
		if( i && i.weapon_info && i.weapon_info.range > 15000 ) return true;
	}
	return false;
}

this._Inject = function(target) {this._Inject2(target, this.ship);}

this._Inject2 = function(target, ship) { //can use injectors and farther than 10km?
	if(ship && target && target.isValid && ship.fuel > 0
		&& ship.equipmentStatus("EQ_FUEL_INJECTION") == "EQUIPMENT_OK"
		&& ship.position.distanceTo(target.position) > 10000 ) {

		//more conditions: not enogh good accuracy, faster target or short range laser?
		if ( ship.accuracy < 7.5 || target.speed > ship.maxspeed )
			return true;
		var l = ship.forwardWeapon.equipmentKey;
		if( l == "EQ_WEAPON_PULSE_LASER" ) return true;
		else if( l ) {  //Oolite 1.81 can define custom lasers
			var i = EquipmentInfo.infoForKey(l);
			if( i && i.weapon_info && i.weapon_info.range < 15000 ) return true;
		}
	}
	return false;
}

this._playerTarget = function() {this._playerTarget2(this.ship);}

this._playerTarget2 = function(ship) {
	var t = player.ship.target;
	if( t && t.isValid && !t.isDerelict && ( !t.script || !( t.script.$EscortDeck_Usable > 0 ) )
		&& ( t.bounty > 2 || t.script && t.script.$EscortDeck_Target ) //nonclean or player fired it
		&& ship.position.distanceTo(t.position) < ship.scannerRange )
		return(t);
	return(null);
}


//ship script events
this.shipCollided = function(otherShip) {
	var w = worldScripts.escortdeck;
	if( otherShip == player.ship && this.ship.energy < 50
		&& !this.ship.addCollisionException //before Oolite 1.81
		&& w.$EscortDeckShip.indexOf(this.ship) > -1 )
		this.ship.energy = 50; //prevent kill owned escorts by accident
}

this.shipTargetDestroyed = function(target) {
	if(this.ship == player.ship) return; //exit if worldScript, do in ship script only
	if(target.primaryRole == "constrictor" && missionVariables.conhunt && missionVariables.conhunt == "STAGE_1") {
		// just in case an escort kills the constrictor, let's not break the mission for the player...
		missionVariables.conhunt = "CONSTRICTOR_DESTROYED";
	}
	
	if(target.isRock || target.isBoulder || target.isCargo || target.isDerelict || target.isWeapon)
		return;
	
//	player.score += 1; - score should remain personal
	player.credits += target.bounty;
	if( this.$EscortDeckUsable > 0 || this.$CarriersShipStorageID > -1 )
		player.consoleMessage(this.ship.displayName+" killed "
			+ target.name + " : " + target.bounty + "₢ awarded.", 5);
	log(this.name, this.ship.displayName+" killed " + target.name + " : " + target.bounty +"₢");
}

this.shipDied = function(whom,why) {
	if(this.ship == player.ship) return; //exit if worldScript, do in ship script only
	var w = worldScripts.escortdeck;
	var pad = w.$EscortDeckShip.indexOf(this.ship);
	if( pad > -1 ) w.$EscortDeckShipData[pad] = null; //prevent recreation in dock
	
	if( whom && whom.isValid && (this.$EscortDeckUsable > 0
		|| this.$CarriersShipStorageID > -1 )) {
		player.commsMessage(this.ship.displayName+" terminated.", 10);
	}
	log(this.name, this.ship.displayName+" terminated by "+whom+" "+why);
}